@@ -10,9 +10,7 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 from . import _algorithm
-import spotpy
 import numpy as np
-import time
 
 
 class sceua(_algorithm):
@@ -62,23 +60,10 @@ class sceua(_algorithm):
             * True:  Simulation results will be saved
             * False: Simulation results will not be saved
         """
-
-        if 'alt_objfun' not in kwargs:
-            kwargs['alt_objfun'] = 'rmse'
+        kwargs['optimization_direction'] = 'minimize'
+        kwargs['algorithm_name'] = 'Shuffled Complex Evolution (SCE-UA) algorithm'
         super(sceua, self).__init__(*args, **kwargs)
-
-    def find_min_max(self):
-        randompar = self.parameter()['random']
-        for i in range(1000):
-            randompar = np.column_stack(
-                (randompar, self.parameter()['random']))
-        return np.amin(randompar, axis=1), np.amax(randompar, axis=1)
-    """
-    def simulate(self,params):
-        if self.repeat.phase=='burnin':
-            id,params = params
-            simulations =
-    """
+        
 
     def simulate(self, id_params_tuple):
         """This overwrites the simple wrapper function of _algorithms.py
@@ -88,16 +73,12 @@ class sceua(_algorithm):
         """
         
         if not self.repeat.phase:  # burn-in
-            id, params = id_params_tuple
-            return id, params, self.model(params)
+            return _algorithm.simulate(self, id_params_tuple)
 
         else:  # complex-evolution
-            igs, x, xf, icall, cx, cf, sce_vars = id_params_tuple
-            self.npg, self.nopt, self.ngs, self.nspl, self.nps, self.bl, self.bu, self.status = sce_vars
+            igs, x, xf, cx, cf, sce_vars = id_params_tuple
+            self.npg, self.nopt, self.ngs, self.nspl, self.nps, self.bl, self.bu, self.stochastic_parameters, discarded_runs = sce_vars
             # Partition the population into complexes (sub-populations);
-#            cx=np.zeros((self.npg,self.nopt))
-#            cf=np.zeros((self.npg))
-            # print(igs)
             k1 = np.arange(self.npg, dtype=int)
             k2 = k1 * self.ngs + igs
             cx[k1, :] = x[k2, :]
@@ -113,11 +94,9 @@ class sceua(_algorithm):
                 lcs[0] = 1
                 for k3 in range(1, self.nps):
                     for i in range(1000):
-                        #lpos = 1 + int(np.floor(self.npg+0.5-np.sqrt((self.npg+0.5)**2 - self.npg*(self.npg+1)*np.random.random())))
                         lpos = int(np.floor(
                             self.npg + 0.5 - np.sqrt((self.npg + 0.5)**2 - self.npg * (self.npg + 1) * np.random.random())))
-                        # idx=find(lcs(1:k3-1)==lpos)
-                        # check of element al eens gekozen
+                        # check if the element has already been chosen
                         idx = (lcs[0:k3] == lpos).nonzero()
                         if idx[0].size == 0:
                             break
@@ -125,16 +104,14 @@ class sceua(_algorithm):
                 lcs.sort()
 
                 # Construct the simplex:
-                s = np.zeros((self.nps, self.nopt))
                 s = cx[lcs, :]
                 sf = cf[lcs]
 
-                snew, fnew, icall, simulation = self._cceua(s, sf, icall)
+                snew, fnew, simulation, discarded_runs = self._cceua(s, sf, discarded_runs)
                 likes.append(fnew)
                 pars.append(snew)
-                #self.status(igs, -fnew, snew)
                 sims.append(simulation)
-                #self.datawriter.save(-fnew,list(snew), simulations = list(simulation),chains = igs)
+                
                 # Replace the worst point in Simplex with the new point:
                 s[-1, :] = snew
                 sf[-1] = fnew
@@ -147,10 +124,9 @@ class sceua(_algorithm):
                 idx = np.argsort(cf)
                 cf = np.sort(cf)
                 cx = cx[idx, :]
-
+                
             # Replace the complex back into the population;
-
-            return igs, likes, pars, sims, cx, cf, k1, k2
+            return igs, likes, pars, sims, cx, cf, k1, k2, discarded_runs
 
     def sample(self, repetitions, ngs=20, kstop=100, pcento=0.0000001, peps=0.0000001):
         """
@@ -171,11 +147,7 @@ class sceua(_algorithm):
         peps: float
             Value of the normalized geometric range of the parameters in the population below which convergence is deemed achieved.
         """
-        print('Starting the SCE-UA algorithm with '+str(repetitions)+ ' repetitions...')
         self.set_repetiton(repetitions)
-        # Initialize the Progress bar
-        starttime = time.time()
-        #intervaltime = starttime
         # Initialize SCE parameters:
         self.ngs = ngs
         randompar = self.parameter()['random']
@@ -185,136 +157,114 @@ class sceua(_algorithm):
         self.nspl = self.npg
         npt = self.npg * self.ngs
         self.iseed = 1
+        self.discarded_runs = 0
         self.bl, self.bu = self.parameter()['minbound'], self.parameter()[
             'maxbound']
         bound = self.bu - self.bl  # np.array
+        self.stochastic_parameters = bound != 0
+        proceed = True
         if self.breakpoint == 'read' or self.breakpoint == 'readandwrite':
             data_frombreak = self.read_breakdata(self.dbname)
             icall = data_frombreak[0]
             x = data_frombreak[1][0]
             xf = data_frombreak[1][1]
             gnrng = data_frombreak[2]
-            acttime = time.time()
+
         elif self.breakpoint is None or self.breakpoint == 'write':
             # Create an initial population to fill array x(npt,self.self.nopt):
             x = self._sampleinputmatrix(npt, self.nopt)
-
-            # Set Ininitial parameter position
-            # iniflg=1
-
             nloop = 0
             icall = 0
             xf = np.zeros(npt)
 
-            print ('burn-in sampling started...')
+            print ('Starting burn-in sampling...')
 
             # Burn in
             param_generator = ((rep, x[rep]) for rep in range(int(npt)))
             for rep, randompar, simulations in self.repeat(param_generator):
                 # Calculate the objective function
-                like = self.postprocessing(icall, randompar, simulations, negativlike=True)
-                
-                #like = self.objectivefunction(
-                #    evaluation=self.evaluation, simulation=simulations)
-                # Save everything in the database
-                
+                like = self.postprocessing(icall, randompar, simulations,chains=0)              
                 xf[rep] = like
-                #self.save(-like, randompar, simulations=simulations)
-                #self.status(rep, -like, randompar)
-                icall += 1
-                # Progress bar
-                #acttime = time.time()
-                #if acttime - intervaltime >= 2:
-                #    text = '%i of %i (best like=%g)' % (
-                #        rep, repetitions, self.status.objectivefunction)
-                #    print(text)
-                #    intervaltime = time.time()
-
+                icall+=1
+                if self.status.stop:
+                    print('Stopping samplig. Maximum number of repetitions reached already during burn-in')
+                    proceed = False
+                    break
             # Sort the population in order of increasing function values;
             idx = np.argsort(xf)
             xf = np.sort(xf)
             x = x[idx, :]
+        else:
+            raise ValueError("Don't know the breakpoint keyword {}".format(self.breakpoint))
         
-        # Record the best and worst points;
+        # Record the best points;
         bestx = x[0, :]
         bestf = xf[0]
-        # worstx=x[-1,:]
-        # worstf=xf[-1]
 
         BESTF = bestf
         BESTX = bestx
-        ICALL = icall
-        # Compute the standard deviation for each parameter
-        # xnstd=np.std(x,axis=0)
 
         # Computes the normalized geometric range of the parameters
         gnrng = np.exp(
-            np.mean(np.log((np.max(x, axis=0) - np.min(x, axis=0)) / bound)))
+            np.mean(np.log((np.max(x[:, self.stochastic_parameters], axis=0) - np.min(x[:, self.stochastic_parameters], axis=0)) / bound[self.stochastic_parameters])))
 
         # Check for convergency;
-        if icall >= repetitions:
+        if self.status.rep >= repetitions:
             print('*** OPTIMIZATION SEARCH TERMINATED BECAUSE THE LIMIT')
             print('ON THE MAXIMUM NUMBER OF TRIALS ')
             print(repetitions)
             print('HAS BEEN EXCEEDED.  SEARCH WAS STOPPED AT TRIAL NUMBER:')
-            print(icall)
+            print(self.status.rep)
             print('OF THE INITIAL LOOP!')
 
         if gnrng < peps:
             print(
                 'THE POPULATION HAS CONVERGED TO A PRESPECIFIED SMALL PARAMETER SPACE')
 
-        print ('burn-in sampling completed...')
+        print ('Burn-in sampling completed...')
 
         # Begin evolution loops:
         nloop = 0
         criter = []
         criter_change_pcent = 1e+5
 
-        #starttime = time.time()
-        #intervaltime = starttime
-        #acttime = time.time()
         self.repeat.setphase('ComplexEvo')
-
-        print ('ComplexEvo started...')
-
-        while icall < repetitions and gnrng > peps and criter_change_pcent > pcento:
+        print ('Starting Complex Evolution...')
+        proceed = True
+        while icall < repetitions and gnrng > peps and criter_change_pcent > pcento and proceed == True:
             nloop += 1
             print ('ComplexEvo loop #%d in progress...' % nloop)
-            # print nloop
-            # print 'Start MPI'
             # Loop on complexes (sub-populations);
             cx = np.zeros((self.npg, self.nopt))
             cf = np.zeros((self.npg))
-
+            remaining_runs = repetitions - self.status.rep
+            if remaining_runs <= self.ngs:
+                self.ngs = remaining_runs-1
+                proceed = False
+            
             sce_vars = [self.npg, self.nopt, self.ngs, self.nspl,
-                        self.nps, self.bl, self.bu, self.status]
-            param_generator = ((rep, x, xf, icall, cx, cf, sce_vars)
+                        self.nps, self.bl, self.bu, self.stochastic_parameters, self.discarded_runs]
+            param_generator = ((rep, x, xf, cx, cf, sce_vars)
                                for rep in range(int(self.ngs)))
-            for igs, likes, pars, sims, cx, cf, k1, k2 in self.repeat(param_generator):
-                icall += len(likes)
+            for igs, likes, pars, sims, cx, cf, k1, k2, discarded_runs in self.repeat(param_generator):
                 x[k2, :] = cx[k1, :]
                 xf[k2] = cf[k1]
-                #print(len(likes))
+                self.discard_runs = discarded_runs
                 for i in range(len(likes)):
-                    #print(icall)
-                    like = self.postprocessing(icall+i, pars[i], sims[i], chains=i, negativlike=True)
-                    #self.save(-likes[i], pars[i],
-                    #                     simulations=sims[i], chains=igs)
-                    #self.status(icall, -likes[i], pars[i])
+                    if not self.status.stop:    
+                        like = self.postprocessing(i, pars[i], sims[i], chains=i+1)
+                    else:
+                        #Collect data from all slaves but do not save
+                        proceed=False
+                        like = self.postprocessing(i, pars[i], sims[i], chains=i+1, save_run=False)
+                        self.discarded_runs+=1
+                        print('Skipping saving')
+                
                 if self.breakpoint == 'write' or self.breakpoint == 'readandwrite'\
-                  and icall >= self.backup_every_rep:
-                    work = (icall, (x, xf), gnrng)
+                  and self.status.rep >= self.backup_every_rep:
+                    work = (self.status.rep, (x, xf), gnrng)
                     self.write_breakdata(self.dbname, work)
 
-                    
-            # Progress bar
-#            acttime = time.time()
-#            if acttime - intervaltime >= 2:
-#                text = '%i of %i (best like=%g)' % (
-#                    icall, repetitions, self.status.objectivefunction)
-#                print(text)
-#                intervaltime = time.time()
             # End of Loop on Complex Evolution;
 
             # Shuffled the complexes;
@@ -325,62 +275,61 @@ class sceua(_algorithm):
             # Record the best and worst points;
             bestx = x[0, :]
             bestf = xf[0]
-            # worstx=x[-1,:]
-            # worstf=xf[-1]
 
             # appenden en op einde reshapen!!
             BESTX = np.append(BESTX, bestx, axis=0)
             BESTF = np.append(BESTF, bestf)
-            ICALL = np.append(ICALL, icall)
 
             # Computes the normalized geometric range of the parameters
             gnrng = np.exp(
-                np.mean(np.log((np.max(x, axis=0) - np.min(x, axis=0)) / bound)))
+                np.mean(np.log((np.max(x[:, self.stochastic_parameters], axis=0) - np.min(x[:, self.stochastic_parameters], axis=0)) / bound[self.stochastic_parameters])))
 
+            criter = np.append(criter, bestf)
+            
             # Check for convergency;
-            if icall >= repetitions:
+            if self.status.rep >= repetitions:
                 print('*** OPTIMIZATION SEARCH TERMINATED BECAUSE THE LIMIT')
                 print('ON THE MAXIMUM NUMBER OF TRIALS ')
                 print(repetitions)
                 print('HAS BEEN EXCEEDED.')
 
-            if gnrng < peps:
+            elif gnrng < peps:
                 print(
                     'THE POPULATION HAS CONVERGED TO A PRESPECIFIED SMALL PARAMETER SPACE')
 
-            criter = np.append(criter, bestf)
 
-            if nloop >= kstop:  # necessary so that the area of high posterior density is visited as much as possible
-                print ('objective function convergence criteria is now being updated and assessed...')
+            elif nloop >= kstop:  # necessary so that the area of high posterior density is visited as much as possible
+                print ('Objective function convergence criteria is now being updated and assessed...')
                 absolute_change = np.abs(
-                    criter[nloop - 1] - criter[nloop - kstop])
+                    criter[nloop - 1] - criter[nloop - kstop])*100
                 denominator = np.mean(np.abs(criter[(nloop - kstop):nloop]))
                 if denominator == 0.0:
                     criter_change_pcent = 0.0
                 else:
-                    criter_change_pcent = absolute_change / denominator * 100
-                print ('updated convergence criteria: %f' % criter_change_pcent)
-                if criter_change_pcent < pcento:
-                    text = 'THE BEST POINT HAS IMPROVED IN LAST %d LOOPS BY LESS THAN THE USER-SPECIFIED THRESHOLD %f' % (
-                        kstop, pcento)
-                    print(text)
+                    criter_change_pcent = absolute_change / denominator
+                print ('Updated convergence criteria: %f' % criter_change_pcent)
+                if criter_change_pcent <= pcento:
+                    print('THE BEST POINT HAS IMPROVED IN LAST %d LOOPS BY LESS THAN THE USER-SPECIFIED THRESHOLD %f' % (
+                        kstop, pcento))
                     print(
                         'CONVERGENCY HAS ACHIEVED BASED ON OBJECTIVE FUNCTION CRITERIA!!!')
+            elif self.status.stop:
+                proceed = False
+                break        
+            
         # End of the Outer Loops
-        text = 'SEARCH WAS STOPPED AT TRIAL NUMBER: %d' % icall
-        print(text)
-        text = 'NORMALIZED GEOMETRIC RANGE = %f' % gnrng
-        print(text)
-        text = 'THE BEST POINT HAS IMPROVED IN LAST %d LOOPS BY %f PERCENT' % (
-            kstop, criter_change_pcent)
-        print(text)
+        print('SEARCH WAS STOPPED AT TRIAL NUMBER: %d' % self.status.rep)
+        print('NUMBER OF DISCARDED TRIALS: %d' % self.discarded_runs)
+        print('NORMALIZED GEOMETRIC RANGE = %f' % gnrng)
+        print('THE BEST POINT HAS IMPROVED IN LAST %d LOOPS BY %f PERCENT' % (
+            kstop, criter_change_pcent))
 
         # reshape BESTX
-        BESTX = BESTX.reshape(BESTX.size // self.nopt, self.nopt)
+        #BESTX = BESTX.reshape(BESTX.size // self.nopt, self.nopt)
         self.final_call()
         
 
-    def _cceua(self, s, sf, icall):
+    def _cceua(self, s, sf, discarded_runs):
             #  This is the subroutine for generating a new point in a simplex
             #
             #   s(.,.) = the sorted simplex in order of increasing function values
@@ -396,7 +345,7 @@ class sceua(_algorithm):
             #   iviol = flag indicating if constraints are violated
             #         = 1 , yes
             #         = 0 , no
-
+        constant_parameters = np.invert(self.stochastic_parameters)
         self.nps, self.nopt = s.shape
         alpha = 1.0
         beta = 0.5
@@ -410,7 +359,7 @@ class sceua(_algorithm):
 
         # Attempt a reflection point
         snew = ce + alpha * (ce - sw)
-
+        snew[constant_parameters] = sw[constant_parameters]
         # Check if is outside the bounds:
         ibound = 0
         s1 = snew - self.bl
@@ -424,43 +373,34 @@ class sceua(_algorithm):
             ibound = 2
 
         if ibound >= 1:
-            snew = self._sampleinputmatrix(1, self.nopt)[0]  # checken!!
+            snew = self._sampleinputmatrix(1, self.nopt)[0]
 
         ##    fnew = functn(self.nopt,snew);
-        simulations = self.model(snew)
-        like = self.postprocessing(icall, snew, simulations, save=False)
-        #like = self.objectivefunction(
-        #    evaluation=self.evaluation, simulation=simulations)
-        # bcf.algorithms._makeSCEUAformat(self.model,self.observations,snew)
+        _, _, simulations = _algorithm.simulate(self, (1, snew))
+        like = self.postprocessing(1, snew, simulations, save_run=False, block_print=True)
+        discarded_runs+=1
+            
         fnew = like
-        #fnew = self.model(snew)
-        icall += 1
 
         # Reflection failed; now attempt a contraction point:
         if fnew > fw:
             snew = sw + beta * (ce - sw)
-            simulations = self.model(snew)
-            like = self.postprocessing(icall, snew, simulations, save=False)
-#            like = self.objectivefunction(
-#                evaluation=self.evaluation, simulation=simulations)
+            snew[constant_parameters] = sw[constant_parameters]
+
+            _, _, simulations = _algorithm.simulate(self, (2, snew))
+            like = self.postprocessing(2, snew, simulations, save_run=False, block_print=True)
+            discarded_runs+=1
             fnew = like
-            icall += 1
 
         # Both reflection and contraction have failed, attempt a random point;
             if fnew > fw:
-                snew = self._sampleinputmatrix(1, self.nopt)[0]  # checken!!
-                simulations = self.model(snew)
-                like = self.postprocessing(icall, snew, simulations, save=False)
-#                like = self.objectivefunction(
-#                    evaluation=self.evaluation, simulation=simulations)
-                # bcf.algorithms._makeSCEUAformat(self.model,self.observations,snew)
-                fnew = like
-                # print 'NSE = '+str((fnew-1)*-1)
-                #fnew = self.model(snew)
-                icall += 1
-
+                snew = self._sampleinputmatrix(1, self.nopt)[0]
+                _, _, simulations = _algorithm.simulate(self, (3, snew))
+                like = self.postprocessing(3, snew, simulations, save_run=False, block_print=True)
+                discarded_runs+=1
+            fnew = like
         # END OF CCE
-        return snew, fnew, icall, simulations
+        return snew, fnew, simulations, discarded_runs
 
     def _sampleinputmatrix(self, nrows, npars):
         '''
@@ -473,8 +413,4 @@ class sceua(_algorithm):
         x = np.zeros((nrows, npars))
         for i in range(nrows):
             x[i, :] = self.parameter()['random']
-        return x
-        # Matrix=np.empty((nrows,npars))
-        # for i in range(nrows):
-        # Matrix[i]= self.parameter()['random']
-        # return Matrix
+        return x
\ No newline at end of file
